@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------------
  * avs2bdnxml - Generates BluRay subtitle stuff from RGBA AviSynth scripts
- * Copyright (C) 2008-2009 Arne Bochem <avs2bdnxml at ps-auxw de>
+ * Copyright (C) 2008-2013 Arne Bochem <avs2bdnxml at ps-auxw de>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,8 +38,14 @@ typedef struct crop_s
 	int h;
 } crop_t;
 
+typedef crop_t rect_t;
+
 void auto_crop (pic_t p, crop_t *c);
-int auto_split (pic_t p, crop_t *c, int ugly);
+int find_windows (crop_t *rects, int n_rects, crop_t *windows);
+int auto_split (pic_t p, crop_t *c, int ugly, int even_y);
+rect_t merge_rects (rect_t r1, rect_t r2);
+int score_rect (rect_t r);
+void enforce_even_y (crop_t *c, int n);
 
 #endif
 
