@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------------
  * avs2bdnxml - Generates BluRay subtitle stuff from RGBA AviSynth scripts
- * Copyright (C) 2008-2010 Arne Bochem <avs2bdnxml at ps-auxw de>
+ * Copyright (C) 2008-2013 Arne Bochem <avs2bdnxml at ps-auxw de>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,6 +15,18 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *----------------------------------------------------------------------------
+ * Version 2.06
+ *   - Add option to add an offset to output timecodes
+ *   - Change return value when no events were detected to 0
+ *   - Add option to allow writing of empty output files
+ *   - Add option to enforce even Y coordinates (may help avoid problems with
+ *     interlaced video in DVD related use cases)
+ *   - Fix SSE2 with more recent GCC version
+ *   - Add option for stricter checks on buffer/palette limits for SUPs
+ *   - Fix bug where LastEventOutTC could be one frame early
+ *   - Add additional checks to commandline argument parsing
+ *   - PNG files are now always placed in the same directory as the XML file
+ *
  * Version 2.05
  *   - Fix crash when using XML and SUP output
  *
@@ -163,8 +175,13 @@ static int64_t gcd( int64_t a, int64_t b )
 #endif
 
 typedef struct {
-#ifndef LINUX
+#if !defined(LINUX)
     PAVISTREAM p_avi;
+#else
+    int fps_den;
+    int fps_num;
+    int frames;
+    FILE *fh;
 #endif
     int width, height;
 } avis_input_t;
@@ -178,8 +195,8 @@ typedef struct {
 
 int open_file_avis( char *psz_filename, avis_input_t **p_handle, stream_info_t *p_param )
 {
-#if !defined(LINUX)
     avis_input_t *h = malloc(sizeof(avis_input_t));
+#if !defined(LINUX)
     AVISTREAMINFO info;
     int i;
 
@@ -227,11 +244,17 @@ int open_file_avis( char *psz_filename, avis_input_t **p_handle, stream_info_t *
 
     return 0;
 #else
+    *p_handle = h;
     p_param->i_width = 1920;
     p_param->i_height = 1080;
     p_param->i_fps_den = 30000;
     p_param->i_fps_num = 1001;
-    *p_handle = (avis_input_t *)fopen(psz_filename, "r");
+    h->width = p_param->i_width;
+    h->height = p_param->i_height;
+    h->fps_den = p_param->i_fps_den;
+    h->fps_num = p_param->i_fps_num;
+    h->fh = fopen(psz_filename, "r");
+    h->frames = 15000;
 		return 0;
 #endif
 }
@@ -247,7 +270,7 @@ int get_frame_total_avis( avis_input_t *handle )
 
     return info.dwLength;
 #else
-    return 6016;
+    return handle->frames;
 #endif
 }
 
@@ -261,7 +284,7 @@ int read_frame_avis( char *p_pic, avis_input_t *handle, int i_frame )
 
     return 0;
 #else
-    fread(p_pic, 4, 1920*1080, (FILE *)handle);
+    fread(p_pic, 4, handle->width * handle->height, handle->fh);
     return 0;
 #endif
 }
@@ -284,7 +307,32 @@ int close_file_avis( avis_input_t *handle )
 
 /* Main avs2bdnxml code starts here, too */
 
-void write_png(int file_id, uint8_t *image, int w, int h, int graphic, uint32_t *pal, crop_t c)
+void get_dir_path(char *filename, char *dir_path)
+{
+	char abs_path[MAX_PATH + 1] = {0};
+	char drive[3] = {0};
+	char dir[MAX_PATH + 1] = {0};
+
+	/* Get absolute path of output XML file */
+	if (_fullpath(abs_path, filename, MAX_PATH) == NULL)
+	{
+		fprintf(stderr, "Cannot determine absolute path for: %s\n", filename);
+		exit(1);
+	}
+
+	/* Split absolute path into components */
+	_splitpath(abs_path, drive, dir, NULL, NULL);
+	strncpy(dir_path, drive, 2);
+	strncat(dir_path, dir, MAX_PATH - 2);
+
+	if (strlen(dir_path) > MAX_PATH - 16)
+	{
+		fprintf(stderr, "Path for PNG files too long.\n");
+		exit(1);
+	}
+}
+
+void write_png(char *dir, int file_id, uint8_t *image, int w, int h, int graphic, uint32_t *pal, crop_t c)
 {
 	FILE *fh;
 	png_structp png_ptr;
@@ -292,13 +340,16 @@ void write_png(int file_id, uint8_t *image, int w, int h, int graphic, uint32_t
 	png_bytep *row_pointers;
 	png_colorp palette = NULL;
 	png_bytep trans = NULL;
-	char filename[15];
+	char tmp[16] = {0};
+	char filename[MAX_PATH + 1] = {0};
 	char *col;
 	int step = pal == NULL ? 4 : 1;
 	int colors = 0;
 	int i;
 
-	snprintf(filename, 15, "%08d_%d.png", file_id, graphic);
+	snprintf(tmp, 15, "%08d_%d.png", file_id, graphic);
+	strncpy(filename, dir, MAX_PATH);
+	strncat(filename, tmp, 15);
 
 	if ((fh = fopen(filename, "wb")) == NULL)
 	{
@@ -387,18 +438,18 @@ void write_png(int file_id, uint8_t *image, int w, int h, int graphic, uint32_t
 	fclose(fh);
 }
 
-int is_identical_sse2 (stream_info_t *s_info, char *img, char *img_old)
+int is_identical_sse2 (stream_info_t *s_info, char volatile *img, char volatile *img_old)
 {
-	char *max = img + s_info->i_width * s_info->i_height * 4;
-	char zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
-	char a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
-	char full[16] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
+	char volatile *max = img + s_info->i_width * s_info->i_height * 4;
+	char volatile zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+	char volatile a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
+	char volatile full[16] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
 	int volatile different = 0;
 	int volatile *diff_ptr = &different;
 
-	asm __volatile__ ("movupd (%%eax), %%xmm3\n" : : "a" (zero));
-	asm __volatile__ ("movupd (%%ecx), %%xmm4\n" : : "c" (a_mask));
-	asm __volatile__ ("movupd (%%edx), %%xmm7\n" : : "d" (full));
+	asm __volatile__ ("movupd (%0), %%xmm3\n" : : "r" (zero));
+	asm __volatile__ ("movupd (%0), %%xmm4\n" : : "r" (a_mask));
+	asm __volatile__ ("movupd (%0), %%xmm7\n" : : "r" (full));
 	while (img < max && !different)
 	{
 		asm __volatile__
@@ -415,6 +466,7 @@ int is_identical_sse2 (stream_info_t *s_info, char *img, char *img_old)
 			"movd %%xmm0, (%%edx)\n"
 			: "=a" (img), "=d" (diff_ptr)
 			: "a" (img), "S" (img_old), "d" (diff_ptr)
+			: "memory"
 		);
 		img += 16;
 		img_old += 16;
@@ -426,16 +478,16 @@ int is_identical_sse2 (stream_info_t *s_info, char *img, char *img_old)
 	return 1;
 }
 
-int is_empty_sse2 (stream_info_t *s_info, char *img)
+int is_empty_sse2 (stream_info_t *s_info, char volatile *img)
 {
-	char *max = img + s_info->i_width * s_info->i_height * 4;
-	char zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
-	char a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
+	char volatile *max = img + s_info->i_width * s_info->i_height * 4;
+	char volatile zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+	char volatile a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
 	int volatile not_empty = 0;
 	int volatile *ne_ptr = &not_empty;
 
-	asm __volatile__ ("movupd (%%eax), %%xmm1\n" : : "a" (zero));
-	asm __volatile__ ("movupd (%%ecx), %%xmm2\n" : : "c" (a_mask));
+	asm __volatile__ ("movupd (%0), %%xmm1\n" : : "r" (zero));
+	asm __volatile__ ("movupd (%0), %%xmm2\n" : : "r" (a_mask));
 	while (img < max && !not_empty)
 	{
 		asm __volatile__
@@ -446,6 +498,7 @@ int is_empty_sse2 (stream_info_t *s_info, char *img)
 			"movd %%xmm0, (%%edx)\n"
 			: "=d" (ne_ptr)
 			: "S" (img), "d" (ne_ptr)
+			: "memory"
 		);
 		img += 16;
 	}
@@ -456,16 +509,16 @@ int is_empty_sse2 (stream_info_t *s_info, char *img)
 	return 1;
 }
 
-void zero_transparent_sse2 (stream_info_t *s_info, char *img)
+void zero_transparent_sse2 (stream_info_t *s_info, char volatile *img)
 {
-	char *max = img + s_info->i_width * s_info->i_height * 4;
-	char zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
-	char a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
-	char full[16] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
+	char volatile *max = img + s_info->i_width * s_info->i_height * 4;
+	char volatile zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+	char volatile a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
+	char volatile full[16] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
 
-	asm __volatile__ ("movupd (%%eax), %%xmm1\n" : : "a" (zero));
-	asm __volatile__ ("movupd (%%ecx), %%xmm2\n" : : "c" (a_mask));
-	asm __volatile__ ("movupd (%%edx), %%xmm3\n" : : "d" (full));
+	asm __volatile__ ("movupd (%0), %%xmm1\n" : : "r" (zero));
+	asm __volatile__ ("movupd (%0), %%xmm2\n" : : "r" (a_mask));
+	asm __volatile__ ("movupd (%0), %%xmm3\n" : : "r" (full));
 	while (img < max)
 	{
 		asm __volatile__
@@ -484,19 +537,19 @@ void zero_transparent_sse2 (stream_info_t *s_info, char *img)
 	}
 }
 
-void swap_rb_sse2 (stream_info_t *s_info, char *img, char *out)
+void swap_rb_sse2 (stream_info_t *s_info, char volatile *img, char volatile *out)
 {
-	char *max = img + s_info->i_width * s_info->i_height * 4;
-	char ag_mask[16] = {0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff};
-	char rb_mask[16] = {0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0};
+	char volatile *max = img + s_info->i_width * s_info->i_height * 4;
+	char volatile ag_mask[16] = {0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff};
+	char volatile rb_mask[16] = {0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0};
 
 	/* Set up masks */
 	asm volatile
 	(
-		"movupd (%%eax), %%xmm0\n"
-		"movupd (%%ecx), %%xmm5\n"
+		"movupd (%0), %%xmm0\n"
+		"movupd (%1), %%xmm5\n"
 		:
-		: "a" (ag_mask), "c" (rb_mask)
+		: "r" (ag_mask), "r" (rb_mask)
 	);
 
 	/* Convert */
@@ -589,10 +642,10 @@ int detect_sse2 ()
 	unsigned int eax, ebx, ecx, edx;
 
 	/* FIXME: SSE2 broken with new GCC. */
-#if __GNUC__ != 3
+/*#if __GNUC__ != 3
 #warning "Disabling SSE2 assembly."
 	return 0;
-#endif
+#endif*/
 
 	if (detection != -1)
 		return detection;
@@ -680,6 +733,7 @@ void mk_timecode (int frame, int fps, char *buf) /* buf must have length 12 (inc
 void print_usage ()
 {
 	fprintf(stderr,
+		"avs2bdnxml 2.06\n\n"
 		"Usage: avs2bdnxml [options] -o output input\n\n"
 		"Input has to be an AviSynth script with RGBA as output colorspace\n\n"
 		"  -o, --output <string>        Output file in BDN XML format\n"
@@ -693,11 +747,18 @@ void print_usage ()
 		"  -f, --fps <float>            Either of: 23.976, 24, 25, 29.97, 50, 59.94\n"
 		"  -x, --x-offset <integer>     X offset, for use with partial frames.\n"
 		"  -y, --y-offset <integer>     Y offset, for use with partial frames.\n"
+		"  -d, --t-offset <string>      Offset timecodes by this many frames or\n"
+		"                               given non-drop timecode (HH:MM:SS:FF).\n"
 		"  -s, --split-at <integer>     Split events longer than this, in frames.\n"
 		"                               Disabled when 0, which is the default.\n"
 		"  -m, --min-split <integer>    Minimum length of line segment after split.\n"
+		"  -e, --even-y <integer>       Enforce even Y coordinates. [on=1, off=0]\n"
 		"  -a, --autocrop <integer>     Automatically crop output. [on=1, off=0]\n"
 		"  -p, --palette <integer>      Output 8bit palette PNG. [on=1, off=0]\n"
+		"  -n, --null-xml <integer>     Allow output of empty XML files. [on=1, off=0]\n"
+		"  -z, --stricter <integer>     Stricter checks in the SUP writer. May lead to\n"
+		"                               less optimized buffer use, but might raise\n"
+		"                               compatibility. [on=1, off=0]\n"
 		"  -u, --ugly <integer>         Allow splitting images in ugly ways.\n"
 		"                               Might improve buffer problems, but is ugly.\n"
 		"                               [on=1, off=0]\n"
@@ -705,7 +766,7 @@ void print_usage ()
 		"                               splitting. [on=1, off=0]\n\n"
 		"Example:\n"
 		"  avs2bdnxml -t Undefined -l und -v 1080p -f 23.976 -a1 -p1 -b0 -m3 \\\n"
-		"    -u0 -o output.xml input.avs\n"
+		"    -u0 -e0 -n0 -z0 -o output.xml input.avs\n"
 		"  (Input and output are required settings. The rest are set to default.)\n"
 		);
 }
@@ -725,6 +786,59 @@ int is_extension(char *filename, char *check_ext)
 	return 0;
 }
 
+int parse_int(char *in, char *name, int *error)
+{
+	char *end;
+	int r;
+	errno = 0;
+	if (error != NULL)
+		*error = 0;
+	r = strtol(in, &end, 10);
+	if (errno || end == in || end != in + strlen(in))
+	{
+		if (error != NULL)
+			*error = 1;
+		if (name != NULL)
+		{
+			fprintf(stderr, "Error: Failed to parse integer (%s): %s\n", name, in);
+			exit(1);
+		}
+	}
+	return r;
+}
+
+int parse_tc(char *in, int fps)
+{
+	int r = 0;
+	int e;
+	int h, m, s, f;
+
+	/* Test for raw frame number. */
+	r = parse_int(in, NULL, &e);
+	if (!e)
+		return r;
+
+	if (strlen(in) != 2 * 4 + 3 || in[2] != ':' || in[5] != ':' || in[8] != ':')
+	{
+		fprintf(stderr, "Error: Invalid timecode offset. Expected FRAMENUMBER or HH:MM:SS:FF, but got: %s\n", in);
+		exit(1);
+	}
+	in[2] = 0;
+	in[5] = 0;
+	in[7] = 0;
+	h = parse_int(in,     "t-offset hours",   NULL);
+	m = parse_int(in + 3, "t-offset minutes", NULL);
+	s = parse_int(in + 6, "t-offset seconds", NULL);
+	f = parse_int(in + 9, "t-offset frames",  NULL);
+	r += f;
+	r += s * fps;
+	fps *= 60;
+	r += m * fps;
+	fps *= 60;
+	r += h * fps;
+	return r;
+}
+
 typedef struct event_s
 {
 	int image_number;
@@ -768,22 +882,22 @@ void add_event_xml (event_list_t *events, int split_at, int min_split, int start
 	}
 }
 
-void write_sup_wrapper (sup_writer_t *sw, uint8_t *im, int num_crop, crop_t *crops, uint32_t *pal, int start, int end, int split_at, int min_split)
+void write_sup_wrapper (sup_writer_t *sw, uint8_t *im, int num_crop, crop_t *crops, uint32_t *pal, int start, int end, int split_at, int min_split, int stricter)
 {
 	int d = end - start;
 
 	if (!split_at)
-		write_sup(sw, im, num_crop, crops, pal, start, end);
+		write_sup(sw, im, num_crop, crops, pal, start, end, stricter);
 	else
 	{
 		while (d >= split_at + min_split)
 		{
 			d -= split_at;
-			write_sup(sw, im, num_crop, crops, pal, start, start + split_at);
+			write_sup(sw, im, num_crop, crops, pal, start, start + split_at, stricter);
 			start += split_at;
 		}
 		if (d)
-			write_sup(sw, im, num_crop, crops, pal, start, start + d);
+			write_sup(sw, im, num_crop, crops, pal, start, start + d, stricter);
 	}
 }
 
@@ -822,17 +936,22 @@ int main (int argc, char *argv[])
 	char *xml_output_fn = NULL;
 	char *x_offset = "0";
 	char *y_offset = "0";
+	char *t_offset = "0";
 	char *buffer_optimize = "0";
 	char *split_after = "0";
 	char *minimum_split = "3";
 	char *palletize_png = "1";
+	char *even_y_string = "0";
 	char *auto_crop_image = "1";
 	char *ugly_option = "0";
 	char *seek_string = "0";
+	char *allow_empty_string = "0";
+	char *stricter_string = "0";
 	char *count_string = "2147483647";
 	char *in_img = NULL, *old_img = NULL, *tmp = NULL, *out_buf = NULL;
 	char *intc_buf = NULL, *outtc_buf = NULL;
 	char *drop_frame = NULL;
+	char png_dir[MAX_PATH + 1] = {0};
 	crop_t crops[2];
 	pic_t pic;
 	uint32_t *pal = NULL;
@@ -842,7 +961,7 @@ int main (int argc, char *argv[])
 	int split_at = 0;
 	int min_split = 3;
 	int autocrop = 0;
-	int xo, yo;
+	int xo, yo, to;
 	int fps = 25;
 	int count_frames = INT_MAX, last_frame;
 	int init_frame = 0;
@@ -853,6 +972,7 @@ int main (int argc, char *argv[])
 	int have_line = 0;
 	int must_zero = 0;
 	int checked_empty;
+	int even_y = 0;
 	int auto_cut = 0;
 	int pal_png = 1;
 	int ugly = 0;
@@ -862,6 +982,8 @@ int main (int argc, char *argv[])
 	int fps_num = 25, fps_den = 1;
 	int sup_output = 0;
 	int xml_output = 0;
+	int allow_empty = 0;
+	int stricter = 0;
 	sup_writer_t *sw = NULL;
 	avis_input_t *avis_hnd;
 	stream_info_t *s_info = malloc(sizeof(stream_info_t));
@@ -887,17 +1009,21 @@ int main (int argc, char *argv[])
 			, {"fps",          required_argument, 0, 'f'}
 			, {"x-offset",     required_argument, 0, 'x'}
 			, {"y-offset",     required_argument, 0, 'y'}
+			, {"t-offset",     required_argument, 0, 'd'}
 			, {"split-at",     required_argument, 0, 's'}
 			, {"min-split",    required_argument, 0, 'm'}
 			, {"autocrop",     required_argument, 0, 'a'}
+			, {"even-y",       required_argument, 0, 'e'}
 			, {"palette",      required_argument, 0, 'p'}
 			, {"buffer-opt",   required_argument, 0, 'b'}
 			, {"ugly",         required_argument, 0, 'u'}
+			, {"null-xml",     required_argument, 0, 'n'}
+			, {"stricter",     required_argument, 0, 'z'}
 			, {0, 0, 0, 0}
 			};
 			int option_index = 0;
 
-			c = getopt_long(argc, argv, "o:j:c:t:l:v:f:x:y:b:s:m:p:a:u:", long_options, &option_index);
+			c = getopt_long(argc, argv, "o:j:c:t:l:v:f:x:y:d:b:s:m:e:p:a:u:n:z:", long_options, &option_index);
 			if (c == -1)
 				break;
 			switch (c)
@@ -935,6 +1061,12 @@ int main (int argc, char *argv[])
 				case 'y':
 					y_offset = optarg;
 					break;
+				case 'd':
+					t_offset = optarg;
+					break;
+				case 'e':
+					even_y_string = optarg;
+					break;
 				case 'p':
 					palletize_png = optarg;
 					break;
@@ -953,6 +1085,12 @@ int main (int argc, char *argv[])
 				case 'u':
 					ugly_option = optarg;
 					break;
+				case 'n':
+					allow_empty_string = optarg;
+					break;
+				case 'z':
+					stricter_string = optarg;
+					break;
 				default:
 					print_usage();
 					return 0;
@@ -986,6 +1124,7 @@ int main (int argc, char *argv[])
 		{
 			xml_output_fn = out_filename[i];
 			xml_output++;
+			get_dir_path(xml_output_fn, png_dir);
 		}
 		else if (is_extension(out_filename[i], "sup") || is_extension(out_filename[i], "pgs"))
 		{
@@ -1001,20 +1140,23 @@ int main (int argc, char *argv[])
 	}
 	if (sup_output > 1 || xml_output > 1)
 	{
-		fprintf(stderr, "If more than one output filename is used, the other must have a\ndifferent output format.\n");
+		fprintf(stderr, "If more than one output filename is used, they must have\ndifferent output formats.\n");
 		exit(0);
 	}
 
 	/* Set X and Y offsets, and split value */
-	xo = atoi(x_offset);
-	yo = atoi(y_offset);
-	pal_png = atoi(palletize_png);
-	autocrop = atoi(auto_crop_image);
-	split_at = atoi(split_after);
-	ugly = atoi(ugly_option);
-	init_frame = atoi(seek_string);
-	count_frames = atoi(count_string);
-	min_split = atoi(minimum_split);
+	xo = parse_int(x_offset, "x-offset", NULL);
+	yo = parse_int(y_offset, "y-offset", NULL);
+	pal_png = parse_int(palletize_png, "palette", NULL);
+	even_y = parse_int(even_y_string, "even-y", NULL);
+	autocrop = parse_int(auto_crop_image, "autocrop", NULL);
+	split_at = parse_int(split_after, "split-at", NULL);
+	ugly = parse_int(ugly_option, "ugly", NULL);
+	allow_empty = parse_int(allow_empty_string, "null-xml", NULL);
+	stricter = parse_int(stricter_string, "stricter", NULL);
+	init_frame = parse_int(seek_string, "seek", NULL);
+	count_frames = parse_int(count_string, "count", NULL);
+	min_split = parse_int(minimum_split, "min-split", NULL);
 	if (!min_split)
 		min_split = 1;
 
@@ -1041,6 +1183,9 @@ int main (int argc, char *argv[])
 		return 1;
 	}
 
+	/* Get timecode offset. */
+	to = parse_tc(t_offset, fps);
+
 	/* Detect CPU features */
 	detect_sse2();
 
@@ -1067,7 +1212,7 @@ int main (int argc, char *argv[])
 	out_buf = out_buf + (short)(16 - ((long)out_buf % 16));
 
 	/* Set up buffer (non-)optimization */
-	buffer_opt = atoi(buffer_optimize);
+	buffer_opt = parse_int(buffer_optimize, "buffer-opt", NULL);
 	pic.b = out_buf;
 	pic.w = s_info->i_width;
 	pic.h = s_info->i_height;
@@ -1146,13 +1291,13 @@ int main (int argc, char *argv[])
 			if (sup_output)
 			{
 				assert(pal != NULL);
-				write_sup_wrapper(sw, (uint8_t *)out_buf, n_crop, crops, pal, start_frame, i, split_at, min_split);
+				write_sup_wrapper(sw, (uint8_t *)out_buf, n_crop, crops, pal, start_frame + to, i + to, split_at, min_split, stricter);
 				if (!xml_output)
 					free(pal);
 				pal = NULL;
 			}
 			if (xml_output)
-				add_event_xml(events, split_at, min_split, start_frame, i, n_crop, crops);
+				add_event_xml(events, split_at, min_split, start_frame + to, i + to, n_crop, crops);
 			end_frame = i;
 			have_line = 0;
 		}
@@ -1171,20 +1316,22 @@ int main (int argc, char *argv[])
 		start_frame = i;
 		swap_rb(s_info, in_img, out_buf);
 		if (buffer_opt)
-			n_crop = auto_split(pic, crops, ugly);
+			n_crop = auto_split(pic, crops, ugly, even_y);
 		else if (autocrop)
 		{
 			crops[0].x = 0;
 			crops[0].y = 0;
 			crops[0].w = pic.w;
 			crops[0].h = pic.h;
-			auto_crop(pic, &(crops[0]));
+			auto_crop(pic, crops);
 		}
+		if ((buffer_opt || autocrop) && even_y)
+			enforce_even_y(crops, n_crop);
 		if ((pal_png || sup_output) && pal == NULL)
 			pal = palletize(out_buf, s_info->i_width, s_info->i_height);
 		if (xml_output)
 			for (j = 0; j < n_crop; j++)
-				write_png(start_frame, (uint8_t *)out_buf, s_info->i_width, s_info->i_height, j, pal, crops[j]);
+				write_png(png_dir, start_frame, (uint8_t *)out_buf, s_info->i_width, s_info->i_height, j, pal, crops[j]);
 		if (pal_png && xml_output && !sup_output)
 		{
 			free(pal);
@@ -1208,14 +1355,14 @@ int main (int argc, char *argv[])
 		if (sup_output)
 		{
 			assert(pal != NULL);
-			write_sup_wrapper(sw, (uint8_t *)out_buf, n_crop, crops, pal, start_frame, i - 1, split_at, min_split);
+			write_sup_wrapper(sw, (uint8_t *)out_buf, n_crop, crops, pal, start_frame + to, i - 1 + to, split_at, min_split, stricter);
 			if (!xml_output)
 				free(pal);
 			pal = NULL;
 		}
 		if (xml_output)
 		{
-			add_event_xml(events, split_at, min_split, start_frame, i - 1, n_crop, crops);
+			add_event_xml(events, split_at, min_split, start_frame + to, i - 1 + to, n_crop, crops);
 			free(pal);
 			pal = NULL;
 		}
@@ -1233,8 +1380,16 @@ int main (int argc, char *argv[])
 		/* Check if we actually have any events */
 		if (first_frame == -1)
 		{
-			fprintf(stderr, "No events detected. Refusing to write XML file.\n");
-			return 1;
+			if (!allow_empty)
+			{
+				fprintf(stderr, "No events detected. Cowardly refusing to write XML file.\n");
+				return 0;
+			}
+			else
+			{
+				first_frame = 0;
+				end_frame = 0;
+			}
 		}
 
 		/* Initialize timecode buffers */
@@ -1249,8 +1404,8 @@ int main (int argc, char *argv[])
 		}
 
 		/* Write XML header */
-		mk_timecode(first_frame, fps, intc_buf);
-		mk_timecode(end_frame, fps, outtc_buf);
+		mk_timecode(first_frame + to, fps, intc_buf);
+		mk_timecode(end_frame + to + auto_cut, fps, outtc_buf);
 		fprintf(fh, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
 			"<BDN Version=\"0.93\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
 			"xsi:noNamespaceSchemaLocation=\"BD-03-006-0093b BDN File Format.xsd\">\n"
@@ -1261,7 +1416,7 @@ int main (int argc, char *argv[])
 			"<Events LastEventOutTC=\"%s\" FirstEventInTC=\"%s\"\n", track_name, language, video_format, frame_rate, drop_frame, outtc_buf, intc_buf);
 
 		mk_timecode(0, fps, intc_buf);
-		mk_timecode(frames, fps, outtc_buf);
+		mk_timecode(frames + to, fps, outtc_buf);
 		fprintf(fh, "ContentInTC=\"%s\" ContentOutTC=\"%s\" NumberofEvents=\"%d\" Type=\"Graphic\"/>\n"
 			"</Description>\n"
 			"<Events>\n", intc_buf, outtc_buf, num_of_events);
