@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------------
  * avs2bdnxml - Generates BluRay subtitle stuff from RGBA AviSynth scripts
- * Copyright (C) 2008-2010 Arne Bochem <avs2bdnxml at ps-auxw de>
+ * Copyright (C) 2008-2013 Arne Bochem <avs2bdnxml at ps-auxw de>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -534,7 +534,7 @@ void write_subtitle (sup_writer_t *sw, uint8_t **rle, int *rle_len, int num_crop
 	 *   composition by one, increase all further picture values by previous number of
 	 *   composition objects, increase palette by one. Object id in PCSS is 1 for all.
 	 *   Do some stuff to WDS too. Keep it to first value, if stuff still fits in the window?
-	 *   It seems WDS has to stay constant withing a single composition. What a pain.
+	 *   It seems WDS has to stay constant within a single composition. What a pain.
 	 */
 	tick_fac *= ((double)sw->fps_den) / ((double)sw->fps_num);
 
@@ -774,7 +774,7 @@ void close_sup_writer (sup_writer_t *sw)
 
 IMPLEMENT_LIST(si, subtitle_info_t)
 
-void write_sup (sup_writer_t *sw, uint8_t *im, int num_crop, rect_t *crops, uint32_t *pal, int start, int end)
+void write_sup (sup_writer_t *sw, uint8_t *im, int num_crop, rect_t *crops, uint32_t *pal, int start, int end, int strict)
 {
 	rect_t tmp;
 	int buffer_increase;
@@ -784,7 +784,7 @@ void write_sup (sup_writer_t *sw, uint8_t *im, int num_crop, rect_t *crops, uint
 	for (i = 0; i < num_crop; i++)
 		buffer_increase += crops[i].w * crops[i].h + 16;
 	/* Disabled some conditions for now. */
-	if (sw->non_new && ((start > sw->end + 1) || /*(sw->buffer + buffer_increase >= 4 * 1024 * 1024) ||*/ (sw->objects + num_crop > 64) /*-|| (sw->palettes + 1 > 8)*/))
+	if (sw->non_new && ((start > sw->end + 1) || (sw->objects + num_crop > 64) || (strict && ((sw->buffer + buffer_increase >= 4 * 1024 * 1024) || (sw->palettes + 1 > 8)))))
 	{
 #		if DEBUG != 0
 #		warning "DEBUG enabled."
@@ -799,14 +799,26 @@ void write_sup (sup_writer_t *sw, uint8_t *im, int num_crop, rect_t *crops, uint
 				printf("due to number of palettes. %u + %u = %u > %u\n", sw->palette, 1, sw->palette + 1, 8);
 			else
 				printf("for unknown reasons.\n");
+#		else
+		if (strict)
+		{
+			if (sw->buffer + buffer_increase >= 4 * 1024 * 1024)
+			{
+				printf("Warning: Starting new epoch due to buffer overflow (%u -> %u > %u) for event starting at frame %u (including offsets) in stricter mode.\n", sw->buffer, sw->buffer + buffer_increase, 4 * 1024 * 1024, start);
+			}
+			else if (sw->palettes + 1 > 8)
+			{
+				printf("Warning: Starting new epoch due to too many palettes for event starting at frame %u (including offsets) in stricter mode.\n", start);
+			}
+		}
 #		endif
 		write_composition(sw);
 	}
 	sw->non_new = 1;
 	sw->end = end;
 	sw->buffer += buffer_increase;
-	sw->objects += num_crop; /* FIXME? */
-	(sw->palettes)++; /* FIXME: This is dumb. */
+	sw->objects += num_crop;
+	(sw->palettes)++; /* FIXME: It's probably okay not to increase this if the palette is identical to the last, but this would require further testing and possibly additional code so identical palettes are not written to the stream multiple times? */
 
 	if (num_crop > 1)
 	{
